@@ -1,7 +1,8 @@
 package Mojolicious::Plugin::RoutesAuthDBI;
 use Mojo::Base 'Mojolicious::Plugin::Authentication';
+use Mojo::Loader qw(load_class);
 
-our $VERSION = '0.302';
+our $VERSION = '0.400';
 
 my $access;# 
 my $pkg = __PACKAGE__;
@@ -10,7 +11,17 @@ my $conf ;# set on ->registrer
 
 my $fail_auth = {format=>'txt', text=>"Deny at auth step. Please sign in!!!\n"};
 my $fail_auth_cb = sub {shift->render(%$fail_auth);};
-my $fail_access_cb = sub {shift->render(format=>'txt', text=>"You don`t have access on this route(url)!!!\n");};
+my $fail_access_cb = sub {
+  my ($c, $route, $args, $u) = @_;
+  $c->app->log->debug(sprintf "Deny [%s] for user id=[%s]; args=[%s]; defaults=[%s]",
+    $route->pattern->unparsed,
+    $u->{id},
+    $c->dumper($args) =~ s/\s+//gr,
+    $c->dumper($route->pattern->defaults) =~ s/\s+//gr,
+  );
+  $c->render(format=>'txt', text=>"You don`t have access on this route(url)!!!\n");
+  
+};
 
 
 
@@ -36,6 +47,7 @@ sub register {
   $self->SUPER::register($app, $conf->{auth});
   
   $app->routes->add_condition(access => \&access);
+  $access->{'app.routes'} = $app->routes;
   $access->apply_route($app, $_) for @{ $access->db_routes };
   
   if ($conf->{admin}) {
@@ -77,43 +89,114 @@ sub _load_mod {
 
 # 
 sub access {# add_condition
-  my ($route, $c, $captures, $r_hash) = @_;
-  #~ $c->app->log->debug($c->dumper($r_hash));#$route->pattern->defaults
+  my ($route, $c, $captures, $args) = @_;
+  #~ $c->app->log->debug($c->dumper($route));#$route->pattern->defaults
   # 1. по паролю выставить куки
   # 2. по кукам выставить пользователя
   my $meth = $conf->{auth}{current_user_fn};
   my $u = $c->$meth;
   # 3. если не проверять доступ вернуть 1
-  return 1 unless $r_hash->{auth};
+  return 1 unless $args->{auth};
   # не авторизовался
   $conf->{access}{fail_auth_cb}->($c, )
     and return undef
     unless $u;
-  # 4. получить все группы пользователя
-  $u->{roles} ||= $access->load_user_roles($c, $u->{'id'});
-  # 5. по ИДам групп и пользователя проверить доступ
+  #  получить все группы пользователя
+  $access->load_user_roles($u);
+
   my $id2 = [$u->{id}, map($_->{id}, grep !$_->{disable},@{$u->{roles}})];
+  my $id1 = [grep $_, @$args{qw(id route_id action_id controller_id namespace_id)}];
   
-  # Acces to route by refs: routes -> roles -> users
-  ($r_hash->{id} && $access->access_route($c, $r_hash->{id}, $id2))
-    and $c->app->log->debug(sprintf "Access on [%s%s%s%s] for user id=[%s] on request=[%s]", $r_hash->{namespace} ? "$r_hash->{namespace}::" : "", $r_hash->{controller} ? "$r_hash->{controller}->" : "", $r_hash->{action} ? $r_hash->{action} : "", $r_hash->{callback} ? " cb => sub {...}" : "", $u->{id}, $r_hash->{request})
-    and return 1;
+  # explicit acces to route
+  scalar @$id1
+    && $access->access_explicit($id1, $id2)
+    && $c->app->log->debug(sprintf "Allow [%s] for roles=[%s] joined id1=%s; args=[%s]; defaults=%s",
+      $route->pattern->unparsed,
+      $c->dumper($id2) =~ s/\s+//gr,
+      $c->dumper($id1) =~ s/\s+//gr,
+      $c->dumper($args) =~ s/\s+//gr,
+      $c->dumper($route->pattern->defaults) =~ s/\s+//gr,
+    )
+    && return 1;
   
-  # Access to route (may be not in db table) of any actions on controller
-  # Refs: controllers -> roles -> users
-  $r_hash->{controller} ||= $route->pattern->defaults->{controller};
-  $r_hash->{namespace} ||= $route->pattern->defaults->{namespace};
-  ($r_hash->{controller} && $access->access_controller($c, $r_hash, $id2))
-    and $c->app->log->debug(sprintf "Access all actions on [%s%s] for user id=[%s] on request=[%s]", $r_hash->{namespace} ? "$r_hash->{namespace}::" : "", $r_hash->{controller} ? "$r_hash->{controller}" : "",$u->{id}, $r_hash->{request})
-    and return 1;
+  # Access to non db route by role
+  $args->{role}
+    && $access->access_role($args->{role}, $id2)
+    && $c->app->log->debug(sprintf "Allow [%s] by role [%s] joined roles=[%s]",
+      $route->pattern->unparsed,
+      $args->{role},
+    )
+    && return 1;
   
-  # Access to route (not in db table) by role
-  ($r_hash->{role} && $access->access_role($c, $r_hash, $id2))
-    and $c->app->log->debug(sprintf "Access by role [%s] for user id=[%s] on request=[%s]", $r_hash->{role}, $u->{id}, $r_hash->{request})
-    and return 1;
+  # implicit access to non db routes
+  my $controller = $args->{controller} || ucfirst(lc($route->pattern->defaults->{controller}));
+  my $namespace = $args->{namespace} || $route->pattern->defaults->{namespace};
+  if ($controller && !$namespace) {
+    (load_class($_.'::'.$controller) or ($namespace = $_) and last) for @{ $access->{'app.routes'}->namespaces };
+  }
+  $conf->{access}{fail_access_cb}->($c, $route, $args, $u)
+    and return undef
+    unless $controller && $namespace;# failed load class
+
+  $access->access_namespace($namespace, $id2)
+    && $c->app->log->debug(sprintf "Allow [%s] for roles=[%s] by namespace=[%s]; args=[%s]; defaults=[%s]",
+      $route->pattern->unparsed,
+      $c->dumper($id2) =~ s/\s+//gr,
+      $namespace,
+      $c->dumper($args) =~ s/\s+//gr,
+      $c->dumper($route->pattern->defaults) =~ s/\s+//gr,
+    )
+    && return 1;
   
-  $conf->{access}{fail_access_cb}->($c, $route, $r_hash);
-  #~ $c->app->log->debug($c->dumper($r_hash));
+  $access->access_controller($namespace, $controller, $id2)
+    && $c->app->log->debug(sprintf "Allow [%s] for roles=[%s] by namespace=[%s] and controller=[%s]; args=[%s]; defaults=[%s]",
+      $route->pattern->unparsed,
+      $c->dumper($id2) =~ s/\s+//gr,
+      $namespace, $controller,
+      $c->dumper($args) =~ s/\s+//gr,
+      $c->dumper($route->pattern->defaults) =~ s/\s+//gr,
+    )
+    && return 1;
+    
+  # еще раз контроллер, который тут без namespace и в базе без namespace ------> доступ из любого места
+  $args->{namespace} || $route->pattern->defaults->{namespace}
+    || $access->access_controller(undef, $controller, $id2)
+    && $c->app->log->debug(sprintf "Allow [%s] for roles=[%s] by controller=[%s] without namespace on db; args=[%s]; defaults=[%s]",
+      $route->pattern->unparsed,
+      $c->dumper($id2) =~ s/\s+//gr,
+      $controller,
+      $c->dumper($args) =~ s/\s+//gr,
+      $c->dumper($route->pattern->defaults) =~ s/\s+//gr,
+    )
+    && return 1;
+  
+  my $action = $args->{action} || $route->pattern->defaults->{action}
+    or $conf->{access}{fail_access_cb}->($c, $route, $args, $u)
+    and return undef;
+  
+  $access->access_action($namespace, $controller, $action, $id2)
+    && $c->app->log->debug(sprintf "Allow [%s] for roles=[%s] by namespace=[%s] and controller=[%s] and action=[%s]; args=[%s]; defaults=[%s]",
+      $route->pattern->unparsed,
+      $c->dumper($id2) =~ s/\s+//gr,
+      $namespace , $controller, $action,
+      $c->dumper($args) =~ s/\s+//gr,
+      $c->dumper($route->pattern->defaults) =~ s/\s+//gr,
+    )
+    && return 1;
+  
+  # еще раз контроллер, который тут без namespace и в базе без namespace ------> доступ из любого места
+  $args->{namespace} || $route->pattern->defaults->{namespace}
+    && $access->access_action(undef, $controller, $action, $id2)
+    && $c->app->log->debug(sprintf "Allow [%s] for roles=[%s] by (namespace=[any]) controller=[%s] and action=[%s]; args=[%s]; defaults=[%s]",
+      $route->pattern->unparsed,
+      $c->dumper($id2) =~ s/\s+//gr,
+      $controller, $action,
+      $c->dumper($args) =~ s/\s+//gr,
+      $c->dumper($route->pattern->defaults) =~ s/\s+//gr,
+    )
+    && return 1;
+  
+  $conf->{access}{fail_access_cb}->($c, $route, $args, $u);
   return undef;
 }
 
@@ -128,14 +211,15 @@ sub access {# add_condition
 
 ¡ ¡ ¡ ALL GLORY TO GLORIA ! ! !
 
+=head1 Mojolicious::Plugin::RoutesAuthDBI
 
 =head1 NAME
 
 Mojolicious::Plugin::RoutesAuthDBI - Generate routes from sql-table, make authentication and make restrict access (authorization) to route with users/roles tables. Plugin makes an auth operations throught the plugin L<Mojolicious::Plugin::Authentication> on which is based.
 
-=head1 DIAGRAM DB DESIGN
+=head1 DB DESIGN DIAGRAM
 
-See L<https://github.com/mche/Mojolicious-Plugin-RoutesAuthDBI/blob/master/Diagram.svg>
+First of all see L<SVG|https://github.com/mche/Mojolicious-Plugin-RoutesAuthDBI/blob/master/Diagram.svg>  or L<PNG|http://i.imgur.com/sXHF4pU.png>
 
 =head1 SYNOPSIS
 
