@@ -13,11 +13,13 @@ my $sth;#sth hub
 
 =encoding utf8
 
+=head1 Mojolicious::Plugin::RoutesAuthDBI::Access
+
 =head1 NAME
 
 Mojolicious::Plugin::RoutesAuthDBI::Access - Generation routes, authentication and controll access to routes trought sintax of ->over(...), see L<Mojolicious::Routes::Route#over>
 
-=head1 DIAGRAM DB DESIGN
+=head1 DB DESIGN DIAGRAM
 
 See L<https://github.com/mche/Mojolicious-Plugin-RoutesAuthDBI/blob/master/Diagram.svg>
 
@@ -76,9 +78,9 @@ Both above options determining the module which will play as manager of authenti
 
 This callback invoke when request need auth route but authentication was failure.
 
-=item * B<fail_access_cb> = sub {my ($c, $route, $r_hash) = @_;...}
+=item * B<fail_access_cb> = sub {my ($c, $route, $r_hash, $u) = @_;...}
 
-This callback invoke when request need auth route but access was failure. $route - L<Mojolicious::Routes::Route> object, $r_hash - route hashref db item.
+This callback invoke when request need auth route but access was failure. $route - L<Mojolicious::Routes::Route> object, $r_hash - route hashref db item, $u - useer hashref.
 
 =back
 
@@ -112,17 +114,25 @@ Fetch records for apply_routes. Must return arrayref of hashrefs routes.
 
 Fetch records roles for session user. Must return arrayref of hashrefs roles.
 
-=item * B<access_route($self, $c, $id1, $id2)>
+=item * B<access_explicit($self, $c, $id1, $id2)>
 
-Check access to route ($id1) by user roles ids ($id2 arrayref). Must return false for deny access or true - allow access.
+Check access to route ($id1 arrayref - either route id or action id or controller id or namespace id) by roles ids ($id2 arrayref). Must return false for deny access or true - allow access.
 
-=item * B<access_controller($self, $c, $r, $id2)>
+=item * B<access_namespace($self, $c, $namespace, $id2)>
 
-Check access to route by $r->{namespace} and $r->{controller} for user roles ids ($id2 arrayref). Must return false for deny access or true - allow access to all actions of this controller.
+Check implicit access to route by $namespace for user roles ids ($id2 arrayref). Must return false for deny access or true - allow access to all actions of this namespace.
 
-=item * B<access_role($self, $c, $r, $id2)>
+=item * B<access_controller($self, $c, $namespace, $controller, $id2)>
 
-Check access to route by role id|name ($r->{role}) and user roles ids ($id2 arrayref). Must return false for deny access or true - allow access.
+Check implicit access to route by $namespace and $controller for user roles ids ($id2 arrayref). Must return false for deny access or true - allow access to all actions of this controller.
+
+=item * B<access_action($self, $c, $namespace, $controller, $action, $id2)>
+
+Check implicit access to route by $namespace and $controller and $action for user roles ids ($id2 arrayref). Must return false for deny access or true - allow access to this action.
+
+=item * B<access_role($self, $c, $role, $id2)>
+
+Check implicit access to route by $role (id|name) and user roles ids ($id2 arrayref). Must return false for deny access or true - allow access.
 
 =back
 
@@ -144,20 +154,20 @@ it under the same terms as Perl itself.
 =cut
 
 sub init_class {# from plugin! init Class vars
-	my $c = shift;
-	my %args = @_;
+  my $c = shift;
+  my %args = @_;
   $init_conf ||= $c;
-	$c->{dbh} ||= $dbh ||=  $args{dbh};
-	$dbh ||= $c->{dbh};
+  $c->{dbh} ||= $dbh ||=  $args{dbh};
+  $dbh ||= $c->{dbh};
   $c->{pos} ||= $args{pos} || $c->{namespace}.'::POS::Pg';
-	$c->{sth} ||= $sth ||= $args{sth} ||=( bless [$dbh, {}], $c->{namespace}.'::Sth' )->init(pos=>$c->{pos});#sth cache
-	$sth ||= $c->{sth};
-	return $c;
+  $c->{sth} ||= $sth ||= $args{sth} ||=( bless [$dbh, {}], $c->{namespace}.'::Sth' )->init(pos=>$c->{pos});#sth cache
+  $sth ||= $c->{sth};
+  return $c;
 }
 
 sub load_user {# import for Mojolicious::Plugin::Authentication
-	my ($c, $uid) = @_;
-	my $u = $dbh->selectrow_hashref($sth->sth('user'), undef, ($uid, undef));
+  my ($c, $uid) = @_;
+  my $u = $dbh->selectrow_hashref($sth->sth('user'), undef, ($uid, undef));
   $c->app->log->debug("Loading user by id=$uid ". ($u ? 'success' : 'failed'));
   $u->{pass} = '**********************' if $u;
   return $u;
@@ -210,28 +220,43 @@ sub apply_route {# meth in Plugin
 }
 
 sub db_routes {
-  my ($self, $c, ) = @_;
+  my ($self,) = @_;
   $dbh->selectall_arrayref($sth->sth('apply routes'), { Slice => {} },);
 }
 
 sub load_user_roles {
-	my ($self, $c, $uid) = @_;
-	$dbh->selectall_arrayref($sth->sth('user roles'), { Slice => {} }, ($uid));
+  my ($self, $user) = @_;
+  $user->{roles} ||= $dbh->selectall_arrayref($sth->sth('user roles'), { Slice => {} }, ($user->{id}));
 }
 
-sub access_route {
-	my ($self, $c, $id1, $id2,) = @_;
-	return scalar $dbh->selectrow_array($sth->sth('cnt refs'), undef, ($id1, $id2));
+sub access_explicit {# i.e. by refs table
+  my ($self, $id1, $id2,) = @_;
+  return scalar $dbh->selectrow_array($sth->sth('cnt refs'), undef, ($id1, $id2));
 }
 
-sub access_controller {
-	my ($self, $c, $r, $id2,) = @_;
-	return scalar $dbh->selectrow_array($sth->sth('access controller'), undef, ($r->{controller}, $r->{namespace},  $id2));
+
+sub access_namespace {#implicit
+  my ($self, $namespace, $id2,) = @_;
+  return scalar $dbh->selectrow_array($sth->sth('access namespace'), undef, ($namespace, $id2));
 }
 
-sub access_role {
-	my ($self, $c, $r, $id2,) = @_;
-	return scalar $dbh->selectrow_array($sth->sth('access role'), undef, ($r->{role} =~ /\D/ ? (undef, $r->{role}) : ($r->{role}, undef),), $id2);
+sub access_controller {#implicit
+  my ($self, $namespace, $controller, $id2,) = @_;
+  my $c = $dbh->selectrow_hashref($sth->sth('controller'), undef, ( $controller, ($namespace) x 2,))
+    or return undef;
+  $self->access_explicit([$c->{id}], $id2);
+}
+
+sub access_action {#implicit
+  my ($self, $namespace, $controller, $action, $id2,) = @_;
+  my $c = $dbh->selectrow_hashref($sth->sth('controller'), undef, ( $controller, ($namespace) x 2,))
+    or return undef;
+  return scalar $dbh->selectrow_array($sth->sth('access action'), undef, ( $c->{id}, $action, $id2));
+}
+
+sub access_role {#implicit
+  my ($self, $role, $id2,) = @_;
+  return scalar $dbh->selectrow_array($sth->sth('access role'), undef, ($role =~ /\D/ ? (undef, $role) : ($role, undef),), $id2);
 }
 
 1;
\ No newline at end of file
