@@ -6,11 +6,13 @@ __PACKAGE__->instance (__FILE__);
 
 =encoding utf8
 
+=head1 Mojolicious::Plugin::RoutesAuthDBI::POS::Pg
+
 =head1 NAME
 
 Mojolicious::Plugin::RoutesAuthDBI::POS::Pg - POS for PostgreSQL.
 
-=head1 DIAGRAM DB DESIGN
+=head1 DB DESIGN DIAGRAM
 
 See L<https://github.com/mche/Mojolicious-Plugin-RoutesAuthDBI/blob/master/Diagram.svg>
 
@@ -31,7 +33,7 @@ L<DBIx::POS>
 
 =over 4
 
-=item * --------------------------------------------------------------------- 
+=item * B<user> 
 
 =name user
 
@@ -43,7 +45,7 @@ L<DBIx::POS>
   from users
   where id = ? or login=?
 
-=item * --------------------------------------------------------------------- 
+=item * B<apply routes> 
 
 =name apply routes
 
@@ -51,18 +53,20 @@ L<DBIx::POS>
 
 =sql
 
-  select r.*, ac.controller, ac.namespace, ac.action, ac.callback, ac.id as action_id
+  select r.*, ac.controller, ac.namespace, ac.action, ac.callback, ac.id as action_id, ac.controller_id, ac.namespace_id
   from routes r
     join refs rf on r.id=rf.id1
     join 
-    (select a.*, c.controller, c.namespace
+    (select a.*, c.controller, c.id as controller_id, n.namespace, n.id as namespace_id
       from actions a 
       left join refs r on a.id=r.id2
       left join controllers c on c.id=r.id1
+      left join refs r2 on c.id=r2.id2
+      left join namespaces n on n.id=r2.id1
     ) ac on rf.id2=ac.id
   order by r.order_by, r.ts;
 
-=item * --------------------------------------------------------------------- 
+=item * B<user roles> 
 
 =name user roles
 
@@ -77,39 +81,60 @@ L<DBIx::POS>
   where r.id2=?;
   --and coalesce(g.disable, 0::bit) <> 1::bit
 
-=item * --------------------------------------------------------------------- 
+=item * B<cnt refs> 
 
 =name cnt refs
 
-=desc check if ref between id1 and [IDs2] exists
+=desc check if ref between [IDs1] and [IDs2] exists
 
 =sql
 
   select count(*)
   from refs
-  where id1 = ? and id2 = ANY(?);
+  where id1 = any(?) and id2 = ANY(?);
 
-=item * ----------------------------------------------------------------------
+=item * B<access action>
 
-=name access controller
+=name access action
 
-=desc доступ ко всем действиям по имени контроллера
+=desc доступ к действию
 
 =sql
 
-  select count(c.*)
-  from 
-    controllers c
-    join refs r on c.id=r.id1
-    join roles o on r.id2=o.id
+  select count(r.*)
+  from
+    refs rc 
+    join actions a on a.id=rc.id2
+    join refs r on a.id=r.id1
+    ---join roles o on o.id=r.id2
   where
-    lower(c.controller)=lower(?)
-    and c.namespace=?
-    and r.id2=any(?)
-    and coalesce(o.disable, 0::bit) <> 1::bit
+    rc.id1=? ---controller id
+    and a.action=?
+    and r.id2=any(?) --- roles ids
+    ---and coalesce(o.disable, 0::bit) <> 1::bit
   ;
 
-=item * ----------------------------------------------------------------------
+
+=item * B<access namespace>
+
+=name access namespace
+
+=desc доступ ко всем действиям по имени пути
+
+=sql
+
+  select count(n.*)
+  from 
+    namespaces n
+    join refs r on n.id=r.id1
+    ---join roles o on r.id2=o.id
+  where
+    n.namespace=?
+    and r.id2=any(?) --- roles ids
+    ---and coalesce(o.disable, 0::bit) <> 1::bit
+  ;
+
+=item * B<access role>
 
 =name access role
 
@@ -121,6 +146,7 @@ L<DBIx::POS>
   from roles
   where (id = ? or name = ?)
     and id = any(?)
+    and coalesce(disable, 0::bit) <> 1::bit
   ;
 
 =back
@@ -129,7 +155,7 @@ L<DBIx::POS>
 
 =over 4
 
-=item * --------------------------------------------------------------------- 
+=item * B<new user> 
 
 =name new user
 
@@ -140,7 +166,7 @@ L<DBIx::POS>
   insert into users (login, pass) values (?,?)
   returning *;
 
-=item * --------------------------------------------------------------------- 
+=item * B<role> 
 
 =name role
 
@@ -152,7 +178,7 @@ L<DBIx::POS>
   from roles
   where id=? or lower(name)=?
 
-=item * --------------------------------------------------------------------- 
+=item * B<new role> 
 
 =name new role
 
@@ -163,7 +189,7 @@ L<DBIx::POS>
   insert into roles (name) values (?)
   returning *;
 
-=item * --------------------------------------------------------------------- 
+=item * B<dsbl/enbl role> 
 
 =name dsbl/enbl role
 
@@ -174,7 +200,7 @@ L<DBIx::POS>
   update roles set disable=?::bit where id=? or lower(name)=?
   returning *;
 
-=item * --------------------------------------------------------------------- 
+=item * B<ref> 
 
 =name ref
 
@@ -186,7 +212,7 @@ L<DBIx::POS>
   from refs
   where id1=? and id2=?;
 
-=item * --------------------------------------------------------------------- 
+=item * B<new ref> 
 
 =name new ref
 
@@ -198,7 +224,7 @@ L<DBIx::POS>
   returning *;
 
 
-=item * --------------------------------------------------------------------- 
+=item * B<del ref> 
 
 =name del ref
 
@@ -210,21 +236,26 @@ L<DBIx::POS>
   where id1=? and id2=?
   returning *;
 
-=item * --------------------------------------------------------------------- 
+=item * B<controller>
 
 =name controller
 
-=desc
+=desc Не пустой namespace - четко привязанный контроллер, пустой - обязательно не привязанный контроллер
 
 =sql
 
-  select *
-  from controllers
+  select c.*, n.namespace, n.id as namespace_id, n.descr as namespace_descr
+  from
+    controllers c
+    left join refs r on c.id=r.id2
+    left join namespaces n on n.id=r.id1
+  
   where
-    namespace=?
-    and lower(controller)=lower(?)
+    c.controller=?
+    and (n.namespace=? or (?::varchar is null and n.id is null))
+    
 
-=item * --------------------------------------------------------------------- 
+=item * B<new controller> 
 
 =name new controller
 
@@ -232,11 +263,11 @@ L<DBIx::POS>
 
 =sql
 
-  insert into controllers (namespace, controller)
+  insert into controllers (controller, descr)
   values (?,?)
   returning *;
 
-=item * --------------------------------------------------------------------- 
+=item * B<new route> 
 
 =name new route
 
@@ -248,7 +279,7 @@ L<DBIx::POS>
   values (?,?,?,?,?,?,?,?,?)
   returning *;
 
-=item * --------------------------------------------------------------------- 
+=item * B<role users> 
 
 =name role users
 
@@ -262,7 +293,7 @@ L<DBIx::POS>
     join refs r on u.id=r.id2
   where r.id1=?;
 
-=item * --------------------------------------------------------------------- 
+=item * B<role routes> 
 
 =name role routes
 
@@ -277,7 +308,7 @@ L<DBIx::POS>
   where r.id2=?;
 
 
-=item * --------------------------------------------------------------------- 
+=item * B<controllers> 
 
 =name controllers
 
@@ -285,31 +316,48 @@ L<DBIx::POS>
 
 =sql
 
-  select * from controllers;
+  select c.*, n.namespace, n.id as namespace_id, n.descr as namespace_descr
+    from controllers c
+    left join refs r on c.id=r.id2
+    left join namespaces n on n.id=r.id1
+    ;
 
-=item * --------------------------------------------------------------------- 
+=item * B<namespaces>
 
-=name controller
+=name namespaces
 
 =desc
 
 =sql
 
-  select * from controllers where namespace = ? and controller = ?;
+  select *
+  from namespaces;
 
-=item * --------------------------------------------------------------------- 
+=item * B<namespace>
 
-=name new controller
+=name namespace
 
 =desc
 
 =sql
 
-  insert into controllers (namespace, controller) values (?,?)
+  select *
+  from namespaces
+  where namespace = ?;
+
+=item * B<new namespace>
+
+=name new namespace
+
+=desc
+
+=sql
+
+  insert into namespaces (namespace, descr) values (?,?)
   returning *;
 
 
-=item * --------------------------------------------------------------------- 
+=item * B<actions>
 
 =name actions
 
@@ -322,7 +370,7 @@ L<DBIx::POS>
     left join refs r on a.id=r.id2
     left join controllers c on c.id=r.id1
 
-=item * --------------------------------------------------------------------- 
+=item * B<> 
 
 =name
 
